"""
List generator
--------------

This function ignores tree structure and works with lists

>>> values = list_generator(["a", "a*b", "b", "c"], conditions=["b%c==1"])
>>> values # doctest: +SKIP
{'a': -8, 'a*b': -40, 'b': 5, 'c': 4}
"""

from .core.generate import random_generator
from .core.grammar import extract_letters, eval_words

DEFAUTL_CONFIG = {
    "rejected": [0],
    "min_max": (-10, 10),
}


def list_generator(
    template: list[str],
    conditions: list[str] = [],
    global_config: dict = {},
    configs: dict = {},
) -> list[int]:
    """Generate random computed values from the list

    :param rd_variables: list of random variables to generate (can be computed value - "a*b")
    :param conditions: condition over variables
    :param global_config: configuration for all variables
    :param configs: configuration for each variables
    :return: list of generated variables

    :example:
    >>> a, ab, b, c = list_generator(["a", "a*b", "b", "c"])
    >>> a, ab, b, c # doctest: +SKIP
    (5, -20, -4, -3)
    >>> a * b == ab
    True
    >>> ab # doctest: +SKIP
    -20
    >>> a, b # doctest: +SKIP
    5, -4
    >>> a, ab, b, c = list_generator(["a", "a*b", "b", "c"], conditions=["a-b==0"])
    >>> a - b == 0
    True
    >>> a, ab, b, c = list_generator(["a", "a*b", "b", "c"], global_config={"rejected": [2, 3, 5, 7]})
    >>> a not in [2, 3, 5, 7]
    True
    >>> b not in [2, 3, 5, 7]
    True
    >>> c not in [2, 3, 5, 7]
    True
    >>> a, ab, b, c = list_generator(["a", "a*b", "b", "c"], configs={"a": {"rejected": [2, 3, 5, 7]}})
    >>> a not in [2, 3, 5, 7]
    True
    """
    rv = extract_letters(template)
    rv_gen = random_generator(
        rv, conditions, dict(DEFAUTL_CONFIG, **global_config), configs
    )
    generated = eval_words(template, rv_gen)
    return [generated[v] for v in template]
